# Plugins web-API

Some plugins expose web interface, that can be reached by a POD. Plugins provide two base endpoints, one for `health status`, second to query information about `exposed API`. POD acts as a proxy between `client` and the `plugin`.

## POD endpoints

### POST /v4/$OWNER_KEY/plugin/status
```json
{
  "auth": $auth_json,
  "payload": {
      "plugins": []
    }
}
```

Returns statuses for selected plugins, `plugins` array accepts `PluginRun` id's, or the `alias` if there is any set.
If `plugins` array is empty, queries all plugins.
If `plugins` contains invalid `id` or `alias` it's get discarded in the response.


Returns `plugins` map, where key is the `PluginRun` id. The `id` or `alias` can be later used in `/plugin/api` and `/plugin/call` endpoints. The value in the map is the plugin metadata containing `PluginStatus` and the plugin `alias`.
### Example response
```json
{
  "plugins": {
    "4fa6094c-51d6-4874-a135-9017fcdedf18": "running"
  },
  "sharedPlugins": {
    "f54d34fe-f223-4075-b702-a0846431dadb": {
      "status": {
        "running": "daemon"
      },
      "alias": "TwitterTopic"
    }
  }
}
```

### POST /v4/$OWNER_KEY/plugin/api
Returns a `JSON` containing information about exposed plugin public API that can be later called by POD's `POST /v4/$OWNER_KEY/plugin/api/call` endpoint. The response JSON is a collection of plugin endpoint's. Each endpoint holds a property `method` that is a HTTP method, like `GET, POST, PUT, DELETE` etc., and `args` containing names of query parameters or json request body.

Arguments with primitive types, like `int`, `string` are used in query string.

Arguments with complex types, like `tuple`, `list`, `dict`, plugin custom `struct` are part of the json request body.

The `id` - PluginRun id or plugin `alias`, identifies which plugin that call is requested
### Example request
```json
{
  "auth": $auth_json,
  "payload": {
      "id": "string"
    }
}
```
### Example response
```json
{
    "/v1/health":  {
        "method": "GET",
        "args": {},
    },
    "/v1/update_model_parameters":  {
        "method": "POST",
        "args": {
            "t": "tuple",
            "req": "ReqModel",
            "a": "typing.List[~AnyStr]",
            "b": "dict",
            "x": "int",
        },
    },
    "/v1/api":  {
        "method": "GET",
        "args": {},
    },
}

```

### POST /v4/$OWNER_KEY/plugin/api/call
Calls plugin endpoint with given parameters. Request body contains
- `id` - PluginRun id or plugin `alias`, identifies which plugin that call is requested
- `method` - HTTP method (`GET, POST, DELETE`,...) used during the call
- `endpoint` - starting with `/` path to the plugin's endpoint
- `query` - a dictionary holding parameters passed as query string
- `jsonBody` - a dictionary holding parameters passed as `json` body.

The example `/v1/update_model_parameters` endpoint from [/v4/$OWNER_KEY/plugin/api](#post-v4owner_keypluginapi), can be reached by following request:

### Example request
```json
{
    "auth": $auth_json,
    "payload": {
      "id": "4fa6094c-51d6-4874-a135-9017fcdedf18",
      "method": "POST",
      "endpoint": "/v1/echo",

      "query": {
        "x": 1234
      },

      "jsonBody": {
        "a": ["string"],

        "b": {},

        "req": {
            "x": 112233,
            "y": false,
            "d": "2022-10-24T19:38:09.718Z"
        },

        "t": [3,2,1]
      }
    }
  }
```

POD response is `200` despite of the status from the plugin endpoint. The plugin endpoint status is in `statusCode` response property. The `body` keeps plugin response in `JSON` format.

### Example response
```json
{
    "status_code": 200,

    "body": {
      "plugin_parameters": {
          "is_shared": false,
          "type": "classifier",
          "model_info": {
              ....
          }
        }
    },
}

```