# Introduction
Spawning ML models on local machine might be memory and CPU/GPU consuming tasks. In some extreme cases it cannot be done
due to hardware limitations. Newly provided feature to the POD allows to attach to the plugins that run on our `dev` environment, so that heavy work is offloaded to the cloud, while POD, and the frontend app can still be launched locally.

# Quick usage
Run the script:
```
KUBECONFIG=~/.kube/do_dev_config examples/run_tunneling_for_plugins.py --plugin-name twitter-topic-model
```

Where `KUBECONFIG` points to a `dev` cluster configuration

The script accepts also other parameters, they are set to defaults if not overridden:
```
  --namespace NAMESPACE
                        kubernetes namespace (default: dev)
  --plugin-name PLUGIN_NAME
                        name of the plugin on k8s (default: None)
  --plugin-alias PLUGIN_ALIAS
                        name you want to use to identify the attached plugin (default: attached-
                        plugin)
  --plugin-forward-port PLUGIN_FORWARD_PORT
                        port used to forward POD requests to K8S (default: 3000)
  --pod-listening-port POD_LISTENING_PORT
                        port that POD is listening for incoming requests (default: 3030)

```

The script will try to run port forwarding using `kubectl` and then, tunneling using `ngrok`. Both executables must be present in the environment.
## Response
```
POD available on https://ffff-ff-ff-ff-ff.eu.ngrok.io, use that url as callback URL for the plugin
To attach local pod to the plugin, run following command.
Please set the OWNER_KEY value in the request, you can also change the id of the plugin

curl -H "content-type: application/json" \
    $POD_URL/v4/$OWNER_KEY/plugin/attach \
    -d @- << EOF
    {
    "auth": {
        "type": "ClientAuth",
        "databaseKey": "DATABASE_KEY"
    },
    "payload": {
        "id": "attached-plugin",
        "webserverPort": 3000,
        "webserverUrl": "http://localhost"
    }
}
EOF


Port forwarding and tunneling are running. Ctrl + C to terminate them.

```
Run the proposed `curl` command to attach owner with `OWNER_KEY` pod on `POD_URL` address to the plugin, do not kill the script since it's used for bidirectional communication.

It's also possible to establish the connection and do the attach in one go. Following env variables needs to be set:

* POD_URL, usually http://localhost:3030
* OWNER_KEY, hex string of existing account
* DB_KEY, hex string, password used for the database

In such case, the script will call the `curl` request for you.

Example response:
```
running ['/usr/local/bin/kubectl', 'port-forward', 'svc/twitter-topic-model', '3000:80', '-n', 'dev'] ...
b'Forwarding from 127.0.0.1:3000 -> 8080\n'
running ['/usr/local/bin/ngrok', 'http', '3030', '--log=stdout'] ...
POD available on https://ffff-ff-ff-ff-ff.eu.ngrok.io, use that url as callback URL for the plugin
Attaching to the plugin, from http://localhost:3030 for owner e111a478066f406a890857de4e7ee8626f164476aaa5422783ce4a0967ba78e5
"b31eca1b183f6d86b26575e9233af91a"
Port forwarding and tunneling are running. Ctrl + C to terminate them.
```


# Talking to the external plugin
## Attach to the plugin
Pod exposes the endpoint `plugin/attach` which is used to create `PluginRun` item in the database, but not to run the plugin itself. Example request:

```json
{
    "auth": {
        "type": "ClientAuth",
        "databaseKey": "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb"
    },
    "payload": {
    "id": "attached-plugin",
    "webserverPort": 3000,
    "webserverUrl": "http://localhost"
    }
}
```

And execution:
```
curl -H "content-type: application/json" -d "@plugin_attach.json"  $POD_URL/v4/$OWNER_KEY/plugin/attach
```

The `id` might be an hexadecimal identifier, or human-friendly name, your choice. `webserverUrl` parameter is usually set to `localhost`, but in some circumstances scheme might differ (`http` or `https`). The `webserverPort` is the port that will be used for communication with external plugin, it's possible to attach to multiple external plugins, the port number must be unique.

## Status of the external plugin
Check it by `/plugin/status` command, by using the `id`.

Request:
```json
{
  "auth": {
    "type": "ClientAuth",
    "databaseKey": "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb"
  },
  "payload": {
    "plugins": ["attached-plugin"]
  }
}
```
Usage:
```
curl -d "@plugin_status.json"  $POD_URL/v4/$OWNER_KEY/plugin/status | jq
```

Response:
```json
{
  "plugins": {
    "4a33bc3bdf878c21769855f5e5a3e15e": {
      "status": "daemon",
      "alias": "attached-plugin"
    }
  },
  "sharedPlugins": {}
}
```
## Calling the API of the plugin
Use the `/plugin/call` endpoint.

Request:
```json
{
  "auth": {
    "type": "ClientAuth",
    "databaseKey": "..."
  },
  "payload": {
    "id": "attached-plugin",
    "method": "POST",
    "endpoint": "/v1/create_topic_model",
    "query": {
      "max_num_tweets": 500
    },
    "jsonBody": {
      "database_key": "...",
      "owner_key": "...",
      "url": "https://POD_URL"
    }
  }
}
```

Usage:
```
curl  -d "@call_api_TT_TM.json"  $POD_URL/v4/$OWNER_KEY/plugin/api/call | jq
```

Note the `url` parameter. It's crucial to set it to the value returned by `ngrok` or other tunneling application. The `url` is used by the plugin to instantiate the `PodClient` object that's used to do database operations, while the request is processed.


# How it works

1) In order to reach the plugin you need to setup port forwarding using kubectl command
    ```
    kubectl port-forward svc/twitter-topic-model 3000:80 -n dev
    ```
    Note port `3000` must be the same as in attach  request. The command above uses `dev` namespace from `dev` cluster. It can happen this cluster is not your default one, resulting in some cryptic error message. To point on correct cluster use `KUBECONFIG` variable like so:

    ```
    KUBECONFIG=~/.kube/do_dev_config kubectl port-forward svc/twitter-topic-model 3000:80 -n dev
    ```

    Where `do_dev_config` is a configuration file for the cluster. Reach DevOps to get that file if you do not have it already.

2) Last step is to enable operations on local POD to the external plugin. That can be simply done by `ngrok` command (download it from the official [ngrok website](https://ngrok.com/))
    ```
    ngrok http 3030
    ```
    Ngrok requires registration, but it has a free license, enough for our needs. Alternatives to `ngrok` are [localtunnel](https://github.com/localtunnel/localtunnel), and [others](https://www.softwaretestinghelp.com/ngrok-alternatives/)