use libpod::{
    api_model::{Bulk, CreateItem},
    async_db_connection::AsyncTx,
    command_line_interface::CliOptions,
    database_api,
    internal_api::{self, create_edge, create_item_tx},
};
use serde_json::json;

pub async fn prepare_graphql_dataset(tx: &AsyncTx, pod_owner: &str, cli: CliOptions) {
    let mut schema = database_api::get_schema(tx).unwrap();
    // insert age prop to Person
    internal_api::bulk_tx(
        tx,
        &mut schema,
        Bulk {
            create_items: vec![
                serde_json::from_value(json!({
                "type": "ItemPropertySchema",
                "itemType": "Person",
                "propertyName": "age",
                "valueType": "Integer"
                }))
                .unwrap(),
                serde_json::from_value(json!({
                "type": "ItemPropertySchema",
                "itemType": "Person1",
                "propertyName": "age",
                "valueType": "Integer"
                }))
                .unwrap(),
                serde_json::from_value(json!({
                "type": "ItemPropertySchema",
                "itemType": "Person2",
                "propertyName": "age",
                "valueType": "Integer"
                }))
                .unwrap(),
                serde_json::from_value(json!({
                "type": "ItemPropertySchema",
                "itemType": "Person3",
                "propertyName": "age",
                "valueType": "Integer"
                }))
                .unwrap(),
            ],
            update_items: vec![],
            delete_items: vec![],
            create_edges: vec![],
            search: vec![],
        },
        &cli,
        pod_owner,
    )
    .await
    .unwrap();

    // create level 1
    for i in 0..1000 {
        let person = json!({
            "type": "Person1",
            "id": "00000".to_owned() + &i.to_string(),
            "age": i
        });
        let person: CreateItem = serde_json::from_value(person).unwrap();
        create_item_tx(tx, &mut schema, person, pod_owner, &cli)
            .await
            .unwrap();
    }

    // create level 2
    for i in 1000..2000 {
        let person = json!({
            "type": "Person2",
            "id": "00000".to_owned() + &i.to_string(),
            "age": i
        });
        let person: CreateItem = serde_json::from_value(person).unwrap();
        create_item_tx(tx, &mut schema, person, pod_owner, &cli)
            .await
            .unwrap();
    }

    // create level 3
    for i in 2000..12000 {
        let person = json!({
            "type": "Person3",
            "id": "00000".to_owned() + &i.to_string()
        });
        let person: CreateItem = serde_json::from_value(person).unwrap();
        create_item_tx(tx, &mut schema, person, pod_owner, &cli)
            .await
            .unwrap();
    }

    // create schema for connections
    let schema_json = json!({
        "type": "ItemEdgeSchema",
        "edgeName": "friend1",
        "sourceType": "Person1",
        "targetType": "Person2",
    });
    let schema_struct: CreateItem = serde_json::from_value(schema_json).unwrap();
    let result = create_item_tx(
        tx,
        &mut database_api::get_schema(tx).unwrap(),
        schema_struct,
        pod_owner,
        &cli,
    )
    .await;
    assert!(result.is_ok());

    let schema_json = json!({
        "type": "ItemEdgeSchema",
        "edgeName": "friend2",
        "sourceType": "Person2",
        "targetType": "Person3",
    });
    let schema_struct: CreateItem = serde_json::from_value(schema_json).unwrap();
    let result = create_item_tx(
        tx,
        &mut database_api::get_schema(tx).unwrap(),
        schema_struct,
        pod_owner,
        &cli,
    )
    .await;
    assert!(result.is_ok());

    // Get updated schema
    let mut schema = database_api::get_schema(tx).unwrap();

    // create edges level 1 -> level 2
    for i in 0..100 {
        let _edge = {
            let json = json!({
                "_source": "00000".to_owned() + &i.to_string(),
                "_target": "00000".to_owned() + &(i+1000).to_string(),
                "_name": "friend1",
            });
            let parsed = serde_json::from_value(json).unwrap();
            create_edge(tx, &mut schema, parsed).unwrap()
        };
    }
    // create edges level 2 -> level 3
    for i in 0..1000 {
        let _edge = {
            let json = json!({
                "_source": "00000".to_owned() + &((i/10)+1000).to_string(),
                "_target": "00000".to_owned() + &(i+2000).to_string(),
                "_name": "friend2",
            });
            let parsed = serde_json::from_value(json).unwrap();
            create_edge(tx, &mut schema, parsed).unwrap()
        };
    }
}
