extern crate cbindgen;

use std::env;
use std::process::Command;
fn main() {
    let git_describe = Command::new("git")
        .args(["describe", "--tags"])
        .output()
        .expect("Failed to launch process 'git describe'");

    if git_describe.status.success() {
        println!(
            "cargo:rustc-env=GIT_DESCRIBE={}",
            String::from_utf8_lossy(&git_describe.stdout)
        );
    } else {
        panic!(
            "git describe command failed: {}",
            String::from_utf8_lossy(&git_describe.stderr)
        );
    }

    // Build script has access to type of build - debug, or release, reexport that information to the pod binary
    // to be used later for constructing version information
    println!(
        "cargo:rustc-env=PROFILE={}",
        env::var("PROFILE").expect("Cargo did not set PROFILE env variable")
    );

    let crate_dir = env::var("CARGO_MANIFEST_DIR").unwrap();
    cbindgen::Builder::new()
        .with_crate(crate_dir)
        .with_language(cbindgen::Language::C)
        .generate()
        .expect("Unable to generate bindings")
        .write_to_file("../memripod/memripod.h");
}
