use crate::{
    api_model::SendEmail,
    command_line_interface::CliOptions,
    constants::{PLUGIN_EMAIL_FOOTER, PLUGIN_EMAIL_SUBJECT_PREFIX},
    error::Result,
};
use lettre::{transport::smtp::authentication::Credentials, Message, SmtpTransport, Transport};
use tracing::{info, trace};

pub fn send_email(email: SendEmail, cli: &CliOptions) -> Result<()> {
    trace!("Starting to send email: {:?}", email);
    match (
        &cli.email_smtp_relay,
        &cli.email_smtp_user,
        &cli.email_smtp_password,
    ) {
        (Some(relay), Some(user), Some(password)) => {
            let email = Message::builder()
                .from(format!("Memri plugin <{user}>").parse()?)
                .to(email.to.parse()?)
                .subject(format!("{}{}", PLUGIN_EMAIL_SUBJECT_PREFIX, email.subject))
                .body(format!("{}{}", PLUGIN_EMAIL_FOOTER, email.body))?;
            let credentials: Credentials = Credentials::new(user.to_string(), password.to_string());
            let server = SmtpTransport::relay(relay)?
                .port(cli.email_smtp_port)
                .credentials(credentials)
                .timeout(Some(core::time::Duration::from_millis(5000)))
                .build();
            server.send(&email)?;
            Ok(())
        }
        _ => {
            debug_email(email);
            Ok(())
        }
    }
}

fn debug_email(email: SendEmail) {
    info!("Email server not configured, debugging email instead {email:#?}");
}
