import 'dart:ffi';
import 'dart:io';
import 'package:ffi/ffi.dart';
import './bindings.dart';

const DYNAMIC_LIBRARY_FILE_NAME_ANDROID = "liblibpod.so";
const DYNAMIC_LIBRARY_FILE_NAME_MACOS = "liblibpod.dylib";

/// Wraps the native functions and converts specific data types in order to
/// handle C strings.

class PodLib {
  static final PodLibBindings _bindings =
  PodLibBindings(PodLib._loadLibrary());

  static DynamicLibrary _loadLibrary() {
    return Platform.isAndroid
        ? DynamicLibrary.open(DYNAMIC_LIBRARY_FILE_NAME_ANDROID)
        : Platform.isMacOS
        ? DynamicLibrary.open(DYNAMIC_LIBRARY_FILE_NAME_MACOS)
        : DynamicLibrary.process();
  }

  static String bulk(String bulk, String databaseKey, String ownerKey) {
    final bulkPtr = bulk.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.bulk(bulkPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String createAccount(String createUserReq) {
    final createUserReqPtr = createUserReq.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.create_account(createUserReqPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String createEdge(String createEdge, String databaseKey, String ownerKey) {
    final createEdgePtr = createEdge.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.create_edge(createEdgePtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String createItem(String createItem, String databaseKey, String ownerKey) {
    final createItemPtr = createItem.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.create_item(createItemPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String deleteEdge(String deleteEdgeBySourceTarget, String databaseKey, String ownerKey) {
    final deleteEdgeBySourceTargetPtr = deleteEdgeBySourceTarget.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.delete_edge(deleteEdgeBySourceTargetPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String deleteItem(String item, String databaseKey, String ownerKey) {
    final itemPtr = item.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.delete_item(itemPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String deleteUser(String databaseKey, String ownerKey) {
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.delete_user(databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String getEdges(String getEdges, String databaseKey, String ownerKey) {
    final getEdgesPtr = getEdges.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.get_edges(getEdgesPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String getFile(String getFile, String databaseKey, String ownerKey) {
    final getFilePtr = getFile.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.get_file(getFilePtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String getItem(String getItem, String databaseKey, String ownerKey) {
    final getItemPtr = getItem.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.get_item(getItemPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String graphql(String item, String databaseKey, String ownerKey) {
    final itemPtr = item.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.graphql(itemPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String oauth1AccessToken(String oauthAccessTokenPayload) {
    final oauthAccessTokenPayloadPtr = oauthAccessTokenPayload.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.oauth1_access_token(oauthAccessTokenPayloadPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String oauth1RequestToken(String oauthRequestTokenPayload) {
    final oauthRequestTokenPayloadPtr = oauthRequestTokenPayload.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.oauth1_request_token(oauthRequestTokenPayloadPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String oauth2AccessToken(String oauth2AccessTokenRequest, String databaseKey, String ownerKey) {
    final oauth2AccessTokenRequestPtr = oauth2AccessTokenRequest.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.oauth2_access_token(itemPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String oauth2AuthUrl(String oauth2AuthUrlRequest) {
    final oauth2AuthUrlRequestPtr = oauth2AuthUrlRequest.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.oauth2_auth_url(oauth2AuthUrlRequestPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String oauth2Authorize(String oauth2AuthorizeTokenRequest, String databaseKey, String ownerKey) {
    final oauth2AuthorizeTokenRequestPtr = oauth2AuthorizeTokenRequest.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.oauth2_authorize(oauth2AuthorizeTokenRequestPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String search(String search, String databaseKey, String ownerKey) {
    final searchPtr = search.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.search(oauth2AuthorizeTokenRequestPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String sendEmail(String sendEmail, String databaseKey, String ownerKey) {
    final sendEmailPtr = sendEmail.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.send_email(sendEmailPtr, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  static String updateItem(String updateItem, String databaseKey, String ownerKey) {
    final updateItemPtr = updateItem.toNativeUtf8().cast<Int8>();
    final databaseKeyPtr = databaseKey.toNativeUtf8().cast<Int8>();
    final ownerKeyPtr = ownerKey.toNativeUtf8().cast<Int8>();

    // Native call
    final ptrResult = _bindings.update_item(updateItem, databaseKeyPtr, ownerKeyPtr);

    // Cast the result pointer to a Dart string
    final result = ptrResult.cast<Utf8>().toDartString();

    // Clone the given result, so that the original string can be freed
    final resultCopy = "" + result;

    // Free the native value
    PodLib._free(result);

    return resultCopy;
  }

  /// Releases the memory allocated to handle the given (result) value
  static void _free(String value) {
    final ptr = value.toNativeUtf8().cast<Int8>();
    return _bindings.rust_cstr_free(ptr);
  }
}
