package com.memri.pod;

import com.memri.pod.Bulk;
import com.memri.pod.CreateEdge;
import com.memri.pod.CreateItem;
import com.memri.pod.DeleteEdgeBySourceTarget;
import com.memri.pod.EdgeDirection;
import com.memri.pod.GetEdges;
import com.memri.pod.GetFile;
import com.memri.pod.Oauth2AccessTokenRequest;
import com.memri.pod.Oauth2AuthUrlRequest;
import com.memri.pod.Oauth2AuthorizeTokenRequest;
import com.memri.pod.OauthAccessTokenPayload;
import com.memri.pod.OauthRequestTokenPayload;
import com.memri.pod.Platforms;
import com.memri.pod.PodLib;
import com.memri.pod.Search;
import com.memri.pod.SendEmail;
import com.memri.pod.SortOrder;

public class App {
  /**
   * Tests the bulk() method of the PodLib.
   */
  public void testBulk() {
    Bulk bulk = new Bulk();
    CreateItem createItem = new CreateItem("item");
    createItem.addField("key", "value");
    createItem.setId("Unique id");
    bulk.addCreateItem(createItem);

    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.bulk(bulk);
  }

  /**
   * Tests the createAccount() method of the PodLib.
   */
  public void testCreateAccount() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.createAccount();
  }

  /**
   * Tests the createEdge() method of the PodLib.
   */
  public void testCreateEdge() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    CreateEdge createEdge = new CreateEdge("name", "source", "target");

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.createEdge(createEdge);
  }

  /**
   * Tests the createItem() method of the PodLib.
   */
  public void testCreateItem() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    CreateItem createItem = new CreateItem("twitter");
    createItem.addField("field key", "field value");
    createItem.setDateCreated(1679908082L);
    createItem.setDateModified(1679908082L);
    createItem.setDateServerModified(1679908082L);
    createItem.setId("create item id");
    createItem.setRowid(1L);

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.createItem(createItem);
  }

  /**
   * Tests the deleteEdge() method of the PodLib.
   */
  public void testDeleteEdge() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    CreateEdge createEdge = new CreateEdge("name", "source", "target");

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.createEdge(createEdge);

    DeleteEdgeBySourceTarget deleteEdgeBySourceTarget = new DeleteEdgeBySourceTarget(
        "source",
        "target"
    );
    podLib.deleteEdge(deleteEdgeBySourceTarget);
  }

  /**
   * Tests the deleteUser() method of the PodLib.
   */
  public void testDeleteUser() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.deleteUser();
  }

  /**
   * Tests the getEdges() method of the PodLib.
   */
  public void testGetEdges() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    GetEdges getEdges = new GetEdges("item", EdgeDirection.Outgoing, true);

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.getEdges(getEdges);
  }

  /**
   * Tests the getFile() method of the PodLib.
   */
  public void testGetFile() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    GetFile getFile = new GetFile("sha");

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.getFile(getFile);
  }

  /**
   * Tests the oauth1AccessToken() method of the PodLib.
   */
  public void testOauth1AccessToken() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    OauthAccessTokenPayload oauthAccessTokenPayload = new OauthAccessTokenPayload(
        "testoauthToken",
        "testoauthTokenSecret",
        "testoauthVerifier"
    );

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.oauth1AccessToken(oauthAccessTokenPayload);
  }

  /**
   * Tests the oauth1RequestToken() method of the PodLib.
   */
  public void testOauth1RequestToken() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    OauthRequestTokenPayload oauthRequestTokenPayload = new OauthRequestTokenPayload(
        "testsetCallbackUrl",
        "testsetService"
    );

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.oauth1RequestToken(oauthRequestTokenPayload);
  }

  /**
   * Tests the oauth2AccessToken() method of the PodLib.
   */
  public void testOauth2AccessToken() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    PodLib podLib = new PodLib(databaseKey, ownerKey);

    Oauth2AccessTokenRequest oauth2AccessTokenRequest = new Oauth2AccessTokenRequest(
        Platforms.Twitter
    );
    podLib.oauth2AccessToken(oauth2AccessTokenRequest);
  }

  /**
   * Tests the oauth2AuthUrl() method of the PodLib.
   */
  public void testOauth2AuthUrl() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    Oauth2AuthUrlRequest oauth2AuthUrlRequest = new Oauth2AuthUrlRequest(
        Platforms.Twitter,
        "testsetRedirectUri"
    );

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.oauth2AuthUrl(oauth2AuthUrlRequest);
  }

  /**
   * Tests the oauth2Authorize() method of the PodLib.
   */
  public void testOauth2Authorize() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    Oauth2AuthorizeTokenRequest oauth2AuthorizeTokenRequest = new Oauth2AuthorizeTokenRequest(
        "testsetAuthCode",
        "testsetPkceVerifier",
        Platforms.Twitter,
        "testsetRedirectUri"
    );

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.oauth2Authorize(oauth2AuthorizeTokenRequest);
  }

  /**
   * Tests the search() method of the PodLib.
   */
  public void testSearch() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    Search search = new Search(10L, SortOrder.Asc);

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.search(search);
  }

  /**
   * Tests the sendEmail() method of the PodLib.
   */
  public void testSendEmail() {
    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    SendEmail sendEmail = new SendEmail("test setBody", "test setSubject", "test setTo");

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.sendEmail(sendEmail);
  }

  /**
   * Tests the updateItem() method of the PodLib.
   */
  public void testUpdateItem() {
    CreateItem createItem = new CreateItem("twitter");
    createItem.addField("field key", "field value");
    createItem.setDateCreated(1679908082L);
    createItem.setDateModified(1679908082L);
    createItem.setDateServerModified(1679908082L);
    createItem.setId("create item id");
    createItem.setRowid(1L);

    String databaseKey = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb";
    String ownerKey = "0000000000000000000000000000000000000000000000000000000000000000";

    PodLib podLib = new PodLib(databaseKey, ownerKey);
    podLib.createItem(createItem);

    UpdateItem updateItem = new UpdateItem("create item id");
    updateItem.addField("field key 2", "field value 2");
    podLib.updateItem(updateItem);
  }

  /**
   * Runs various test methods of the App class.
   */
  public static void main(String[] args) {
    App app = new App();
    app.testBulk();
    app.testCreateAccount();
    app.testCreateEdge();
    app.testCreateItem();
    app.testDeleteEdge();
    app.testDeleteUser();
    app.testGetEdges();
    app.testGetFile();
    app.testOauth1AccessToken();
    app.testOauth1RequestToken();
    app.testOauth2AccessToken();
    app.testOauth2AuthUrl();
    app.testOauth2Authorize();
    app.testSearch();
    app.testSendEmail();
    app.testUpdateItem();
  }
}
