package com.memri.pod;

import com.memri.pod.CreateEdge;
import com.memri.pod.CreateItem;
import com.memri.pod.Search;
import com.memri.pod.UpdateItem;
import java.util.ArrayList;
import java.util.List;

public class Bulk {
  List<CreateEdge> createEdges;
  List<CreateItem> createItems;
  List<String> deleteItems;
  List<Search> search;
  List<UpdateItem> updateItems;

  /**
   * Constructs a new instance of Bulk.
   */
  public Bulk() {
    createEdges = new ArrayList<CreateEdge>();
    createItems = new ArrayList<CreateItem>();
    deleteItems = new ArrayList<String>();
    search = new ArrayList<Search>();
    updateItems = new ArrayList<UpdateItem>();
  }

  /**
   * Adds a CreateEdge object to the list of createEdges in the Bulk object.
   *
   * @param createEdge the CreateEdge object to add
   * @return the Bulk object with the added CreateEdge object
   */
  public Bulk addCreateEdge(CreateEdge createEdge) {
    createEdges.add(createEdge);

    return this;
  }

  /**
   * Adds a CreateItem object to the list of items to be created in bulk.
   *
   * @param createItem the CreateItem object to add to the list
   * @return the updated Bulk object
   */
  public Bulk addCreateItem(CreateItem createItem) {
    createItems.add(createItem);

    return this;
  }

  /**
   * Adds a delete item request to the bulk operation.
   *
   * @param deleteItem the id of the item to be deleted
   * @return the Bulk object with the delete item request added
   */
  public Bulk addDeleteItem(String deleteItem) {
    deleteItems.add(deleteItem);

    return this;
  }

  /**
   * Adds a search operation to the bulk request.
   *
   * @param addSearch the Search object representing the search operation to add
   * @return the Bulk object for chaining additional operations
   */
  public Bulk addSearch(Search addSearch) {
    search.add(addSearch);

    return this;
  }

  /**
   * Adds an UpdateItem object to the list of update items in this Bulk object.
   *
   * @param updateItem The UpdateItem object to add.
   * @return This Bulk object.
   */
  public Bulk addUpdateItem(UpdateItem updateItem) {
    updateItems.add(updateItem);

    return this;
  }
}
