package com.memri.pod;

import java.util.HashMap;

public class CreateItem {
  Long dateCreated;
  Long dateModified;
  Long dateServerModified;
  boolean deleted;
  HashMap<String, String> fields;
  String id;
  Long rowid;
  String type;

  /**
   * Creates a new CreateItem object with the given type.
   *
   * @param setType the type of the new item
   */
  public CreateItem(String setType) {
    dateCreated = null;
    dateModified = null;
    dateServerModified = null;
    fields = new HashMap<String, String>();
    id = null;
    rowid = null;
    type = setType;
    deleted = false;
  }

  /**
   * Sets the date the item was created.
   *
   * @param setDateCreated the date the item was created.
   * @return The updated CreateItem object.
   */
  public CreateItem setDateCreated(Long setDateCreated) {
    dateCreated = setDateCreated;

    return this;
  }

  /**
   * Sets the date modified of the item.
   *
   * @param setDateModified The date modified of the item.
   * @return The updated CreateItem object.
   */
  public CreateItem setDateModified(Long setDateModified) {
    dateModified = setDateModified;

    return this;
  }

  /**
   * Sets the server-modified date of the created item.
   *
   * @param setDateServerModified the date to set as the server-modified date
   * @return The updated CreateItem object
   */
  public CreateItem setDateServerModified(Long setDateServerModified) {
    dateServerModified = setDateServerModified;

    return this;
  }

  /**
   * Sets the 'deleted' flag of the item to the specified value.
   *
   * @param setDeleted boolean value indicating whether the item is deleted or not
   * @return The updated CreateItem object
   */
  public CreateItem setDeleted(boolean setDeleted) {
    deleted = setDeleted;

    return this;
  }

  /**
   * Adds a field to the item being created.
   *
   * @param key the key of the field to be added
   * @param value the value of the field to be added
   * @return the updated CreateItem object
   */
  public CreateItem addField(String key, String value) {
    fields.put(key, value);

    return this;
  }

  /**
   * Sets the ID of the item being created.
   *
   * @param setId the ID to set for the item
   * @return the updated CreateItem object
   */
  public CreateItem setId(String setId) {
    id = setId;

    return this;
  }

  /**
   * Sets the row ID of the item.
   *
   * @param setRowid the row ID to set
   * @return the updated CreateItem object
   */
  public CreateItem setRowid(Long setRowid) {
    rowid = setRowid;

    return this;
  }

  /**
   * Sets the type of the item.
   *
   * @param setType the type to set for the item.
   * @return the updated CreateItem object
   */
  public CreateItem setType(String setType) {
    type = setType;

    return this;
  }
}
