package com.memri.pod;

import com.memri.pod.Platforms;
import java.util.ArrayList;
import java.util.List;

public class Oauth2AuthUrlRequest {
  Platforms platform;
  String redirectUri;
  List<String> scopes;

  /**
   * Constructs a new instance of Oauth2AuthUrlRequest with the specified platform and redirect URI.
   *
   * @param setPlatform the platform to be used for the OAuth2 authentication URL request
   * @param setRedirectUri the redirect URI to be used for the OAuth2 authentication URL request
   */
  public Oauth2AuthUrlRequest(Platforms setPlatform, String setRedirectUri) {
    platform = setPlatform;
    redirectUri = setRedirectUri;
    scopes = new ArrayList<String>();
  }

  /**
   * Sets the platform to be used for the OAuth2 authentication URL request.
   *
   * @param setPlatform the platform to be used for the OAuth2 authentication URL request
   * @return an instance of Oauth2AuthUrlRequest with the platform set
   */
  public Oauth2AuthUrlRequest setPlatform(Platforms setPlatform) {
    platform = setPlatform;

    return this;
  }

  /**
   * Sets the redirect URI to be used for the OAuth2 authentication URL request.
   *
   * @param setRedirectUri the redirect URI to be used for the OAuth2 authentication URL request
   * @return an instance of Oauth2AuthUrlRequest with the redirect URI set
   */
  public Oauth2AuthUrlRequest setRedirectUri(String setRedirectUri) {
    redirectUri = setRedirectUri;

    return this;
  }

  /**
   * Adds a scope to the OAuth2 authentication URL request.
   *
   * @param scope the scope to be added to the OAuth2 authentication URL request
   * @return an instance of Oauth2AuthUrlRequest with the scope added
   */
  public Oauth2AuthUrlRequest addScope(String scope) {
    scopes.add(scope);

    return this;
  }
}
