package com.memri.pod;

import com.memri.pod.Platforms;

public class Oauth2AuthorizeTokenRequest {
  String authCode;
  String pkceVerifier;
  Platforms platform;
  String redirectUri;

  /**
   * Constructs an Oauth2AuthorizeTokenRequest object with the specified parameters.
   *
   * @param setAuthCode the authorization code received from the authorization server
   * @param setPkceVerifier the PKCE verifier used to generate the code challenge
   * @param setPlatform the platform on which the request is being made
   * @param setRedirectUri the redirect URI specified in the initial authorization request
   */
  public Oauth2AuthorizeTokenRequest(
      String setAuthCode,
      String setPkceVerifier,
      Platforms setPlatform,
      String setRedirectUri
  ) {
    authCode = setAuthCode;
    pkceVerifier = setPkceVerifier;
    platform = setPlatform;
    redirectUri = setRedirectUri;
  }

  /**
   * Sets the authorization code for the OAuth2 authorization token request.
   *
   * @param setAuthCode the authorization code to be set
   * @return the instance of the Oauth2AuthorizeTokenRequest class
   */
  public Oauth2AuthorizeTokenRequest setAuthCode(String setAuthCode) {
    authCode = setAuthCode;

    return this;
  }

  /**
   * Sets the PKCE verifier for the authorization token request.
   *
   * @param setPkceVerifier the PKCE verifier to be set
   * @return the instance of the Oauth2AuthorizeTokenRequest class
   */
  public Oauth2AuthorizeTokenRequest setPkceVerifier(String setPkceVerifier) {
    pkceVerifier = setPkceVerifier;

    return this;
  }

  /**
   * Sets the platform for the authorization token request.
   *
   * @param setPlatform the platform to be set.
   * @return the instance of the Oauth2AuthorizeTokenRequest class
   */
  public Oauth2AuthorizeTokenRequest setPlatform(Platforms setPlatform) {
    platform = setPlatform;

    return this;
  }

  /**
   * Sets the redirect URI for the OAuth2 token request.
   *
   * @param setRedirectUri the redirect URI to set
   * @return this {@code Oauth2AuthorizeTokenRequest} object
   */
  public Oauth2AuthorizeTokenRequest setRedirectUri(String setRedirectUri) {
    redirectUri = setRedirectUri;

    return this;
  }
}
