package com.memri.pod;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.memri.pod.Bulk;
import com.memri.pod.CreateEdge;
import com.memri.pod.CreateItem;
import com.memri.pod.CreateUserReq;
import com.memri.pod.DeleteEdgeBySourceTarget;
import com.memri.pod.GetEdges;
import com.memri.pod.GetFile;
import com.memri.pod.Oauth2AccessTokenRequest;
import com.memri.pod.Oauth2AuthUrlRequest;
import com.memri.pod.Oauth2AuthorizeTokenRequest;
import com.memri.pod.OauthAccessTokenPayload;
import com.memri.pod.OauthRequestTokenPayload;
import com.memri.pod.Search;
import com.memri.pod.SendEmail;
import com.memri.pod.UpdateItem;
import static com.memri.pod.ffi.memripod_h.*;
import jdk.incubator.foreign.MemoryAddress;
import jdk.incubator.foreign.MemorySegment;

public class PodLib {
  String databaseKey;
  String ownerKey;

  public PodLib(String database, String owner) {
    databaseKey = database;
    ownerKey = owner;
  }

  public void bulk(Bulk bulk) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(bulk);
    System.out.println(json);
    System.out.println("bulk");
  }

  public void createAccount() {
    CreateUserReq createUserReq = new CreateUserReq(databaseKey, ownerKey);
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(createUserReq);
    System.out.println("createAccount");
    System.out.println(json);

    MemorySegment memorySegment = MemorySegment.ofArray(json.getBytes());
    MemoryAddress result = create_account(memorySegment);

    System.out.println(result);
  }

  public void createEdge(CreateEdge createEdge) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(createEdge);
    System.out.println(json);
    System.out.println("createEdge");
  }

  public void createItem(CreateItem createItem) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(createItem);
    System.out.println(json);
    System.out.println("createItem");
  }

  public void deleteEdge(DeleteEdgeBySourceTarget deleteEdgeBySourceTarget) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(deleteEdgeBySourceTarget);
    System.out.println(json);

    System.out.println("deleteEdge");
  }

  public void deleteItem() {
    System.out.println("deleteItem");
  }

  public void deleteUser() {
    System.out.println("deleteUser");
  }

  public void getEdges(GetEdges getEdges) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(getEdges);
    System.out.println(json);

    System.out.println("getEdges");
  }

  public void getFile(GetFile getFile) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(getFile);
    System.out.println(json);

    System.out.println("getFile");
  }

  public void getItem() {
    System.out.println("getItem");
  }

  public void graphql() {
    System.out.println("graphql");
  }

  public void oauth1AccessToken(OauthAccessTokenPayload oauthAccessTokenPayload) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(oauthAccessTokenPayload);
    System.out.println(json);

    System.out.println("oauth1AccessToken");
  }

  public void oauth1RequestToken(OauthRequestTokenPayload oauthRequestTokenPayload) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(oauthRequestTokenPayload);
    System.out.println(json);

    System.out.println("oauth1RequestToken");
  }

  public void oauth2AccessToken(Oauth2AccessTokenRequest oauth2AccessTokenRequest) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(oauth2AccessTokenRequest);
    System.out.println(json);

    System.out.println("oauth2AccessToken");
  }

  public void oauth2AuthUrl(Oauth2AuthUrlRequest oauth2AuthUrlRequest) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(oauth2AuthUrlRequest);
    System.out.println(json);

    System.out.println("oauth2AuthUrl");
  }

  public void oauth2Authorize(Oauth2AuthorizeTokenRequest oauth2AuthorizeTokenRequest) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(oauth2AuthorizeTokenRequest);
    System.out.println(json);

    System.out.println("oauth2Authorize");
  }

  public void search(Search search) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(search);
    System.out.println(json);

    System.out.println("search");
  }

  public void sendEmail(SendEmail sendEmail) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(sendEmail);
    System.out.println(json);

    System.out.println("sendEmail");
  }

  public void updateItem(UpdateItem updateItem) {
    GsonBuilder builder = new GsonBuilder();
    Gson gson = builder.create();

    String json = gson.toJson(updateItem);
    System.out.println(json);

    System.out.println("updateItem");
  }
}
