package com.memri.pod;

import com.memri.pod.SortOrder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class Search {
  Long dateServerModifiedGte;
  Long dateServerModifiedLt;
  Long dateServerModified;
  Boolean deleted;
  String id;
  Long limit;
  HashMap<String, String> otherProperties;
  List<String> returnProperties;
  SortOrder sortOrder;
  String type;

  /**
   * Constructs a new Search object with the specified limit and sort order.
   *
   * @param setLimit the maximum number of results to return
   * @param setSortOrder the sort order to apply to the results
   */
  public Search(Long setLimit, SortOrder setSortOrder) {
    dateServerModifiedGte = null;
    dateServerModifiedLt = null;
    dateServerModified = null;
    deleted = null;
    id = null;
    limit = setLimit;
    otherProperties = new HashMap<String, String>();
    returnProperties = new ArrayList<String>();
    sortOrder = setSortOrder;
    type = null;
  }

  /**
   * Sets the value for the "dateServerModifiedGte" field in the Search object.
   *
   * @param setDateServerModifiedGte a Long representing the value for the
   *     "dateServerModifiedGte" field to be set
   * @return This instance of the Search object
   */
  public Search setDateServerModifiedGte(Long setDateServerModifiedGte) {
    dateServerModifiedGte = setDateServerModifiedGte;

    return this;
  }

  /**
   * Sets the dateServerModifiedLt parameter for the search query.
   *
   * @param setDateServerModifiedLt the value to set for the dateServerModifiedLt parameter
   * @return This instance of the Search object
   */
  public Search setDateServerModifiedLt(Long setDateServerModifiedLt) {
    dateServerModifiedLt = setDateServerModifiedLt;

    return this;
  }

  /**
   * Sets the dateServerModified property to the given value.
   *
   * @param setDateServerModified the value to set the dateServerModified property to
   * @return This instance of the Search object
   */
  public Search setDateServerModified(Long setDateServerModified) {
    dateServerModified = setDateServerModified;

    return this;
  }

  /**
   * Sets the value for the "deleted" property in the Search object.
   *
   * @param setDeleted The boolean value to set for the "deleted" property.
   * @return This instance of the Search object.
   */
  public Search setDeleted(Boolean setDeleted) {
    deleted = setDeleted;

    return this;
  }

  /**
   * Sets the ID to search for in the database.
   *
   * @param setId The ID to search for.
   * @return This instance of the Search object.
   */
  public Search setId(String setId) {
    id = setId;

    return this;
  }

  /**
   * Sets the limit for the number of results returned by the search.
   *
   * @param setLimit the limit to set
   * @return This instance of the Search object
   */
  public Search setLimit(Long setLimit) {
    limit = setLimit;

    return this;
  }

  /**
   * Adds a key-value pair to the map of other properties to be used in the search query.
   *
   * @param key the key of the property to be added
   * @param value the value of the property to be added
   * @return This instance of the Search object
   */
  public Search addOtherProperty(String key, String value) {
    otherProperties.put(key, value);

    return this;
  }

  /**
   * Adds a return property to the search query.
   *
   * @param returnProperty The name of the property to be returned in the search results.
   * @return This instance of the Search object.
   */
  public Search addReturnProperty(String returnProperty) {
    returnProperties.add(returnProperty);

    return this;
  }

  /**
   * Sets the sort order for the search.
   *
   * @param setSortOrder the sort order to be set.
   * @return This instance of the Search object.
   */
  public Search setSortOrder(SortOrder setSortOrder) {
    sortOrder = setSortOrder;

    return this;
  }

  /**
   * Sets the value of the type parameter for the search query.
   *
   * @param setType a String representing the type of the object to search for
   * @return This instance of the Search object
   */
  public Search setType(String setType) {
    type = setType;

    return this;
  }
}
