#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

/**
 * # Safety
 * The bulk_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *bulk(const char *bulk_ptr, const char *database_key_ptr, const char *owner_key_ptr);

/**
 * # Safety
 * The create_user_req_ptr must point to a valid String.
 */
const char *create_account(const char *create_user_req_ptr);

/**
 * # Safety
 * The create_edge_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *create_edge(const char *create_edge_ptr,
                        const char *database_key_ptr,
                        const char *owner_key_ptr);

/**
 * # Safety
 * The create_item_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *create_item(const char *create_item_ptr,
                        const char *database_key_ptr,
                        const char *owner_key_ptr);

/**
 * # Safety
 * The delete_edge_by_source_target_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *delete_edge(const char *delete_edge_by_source_target_ptr,
                        const char *database_key_ptr,
                        const char *owner_key_ptr);

/**
 * # Safety
 * The item_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *delete_item(const char *item_ptr,
                        const char *database_key_ptr,
                        const char *owner_key_ptr);

/**
 * # Safety
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *delete_user(const char *database_key_ptr, const char *owner_key_ptr);

/**
 * # Safety
 * The get_edges_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *get_edges(const char *get_edges_ptr,
                      const char *database_key_ptr,
                      const char *owner_key_ptr);

/**
 * # Safety
 * The get_file_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *get_file(const char *get_file_ptr,
                     const char *database_key_ptr,
                     const char *owner_key_ptr);

/**
 * # Safety
 * The item_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *get_item(const char *item_ptr, const char *database_key_ptr, const char *owner_key_ptr);

/**
 * # Safety
 * The item_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *graphql(const char *item_ptr, const char *database_key_ptr, const char *owner_key_ptr);

/**
 * # Safety
 * The oauth_access_token_payload_ptr must point to a valid String.
 */
const char *oauth1_access_token(const char *oauth_access_token_payload_ptr);

/**
 * # Safety
 * The oauth_request_token_payload_ptr must point to a valid String.
 */
const char *oauth1_request_token(const char *oauth_request_token_payload_ptr);

/**
 * # Safety
 * The oauth2_access_token_request_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *oauth2_access_token(const char *oauth2_access_token_request_ptr,
                                const char *database_key_ptr,
                                const char *owner_key_ptr);

/**
 * # Safety
 * The oauth2_auth_url_request_ptr must point to a valid String.
 */
const char *oauth2_auth_url(const char *oauth2_auth_url_request_ptr);

/**
 * # Safety
 * The oauth2_authorize_token_request_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *oauth2_authorize(const char *oauth2_authorize_token_request_ptr,
                             const char *database_key_ptr,
                             const char *owner_key_ptr);

/**
 * # Safety
 * The search_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *search(const char *search_ptr, const char *database_key_ptr, const char *owner_key_ptr);

/**
 * # Safety
 * The send_email_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *send_email(const char *send_email_ptr,
                       const char *database_key_ptr,
                       const char *owner_key_ptr);

/**
 * # Safety
 * The update_item_ptr must point to a valid String.
 * The database_key_ptr must point to a valid String.
 * The owner_key_ptr must point to a valid String.
 */
const char *update_item(const char *update_item_ptr,
                        const char *database_key_ptr,
                        const char *owner_key_ptr);
