mod common;
use crate::common::pod_request::register_account;

use common::test_data::TestData;
use serde_json::json;

use test_context::test_context;

#[test_context(TestData)]
#[tokio::test]
async fn test_cannot_create_item_outside_schema(ctx: &mut TestData) {
    register_account(&ctx.pod_client).await;

    let payload = json!(
    {
        "type": "BlaBlaBla",
        "content": "this is a fake content",
        "dateSent": 1655170839993u64
    });

    let resp = ctx.pod_client.post_to(payload, "create_item").await;
    assert_eq!(resp.status(), reqwest::StatusCode::BAD_REQUEST);
    assert_eq!(
        resp.json::<String>().await.unwrap(),
        "Failure: Error 400 Bad Request, No schema definition for 'BlaBlaBla'"
    );
}

#[test_context(TestData)]
#[tokio::test]
async fn test_cannot_create_item_with_props_outside_schema(ctx: &mut TestData) {
    register_account(&ctx.pod_client).await;

    let payload = json!(
    {
        "type": "PluginRun",
        "notInSchema": "this is a fake content",
        "containerImage": "this is a fake content"

    });

    let resp = ctx.pod_client.post_to(payload, "create_item").await;
    assert_eq!(resp.status(), reqwest::StatusCode::BAD_REQUEST);
    assert!(resp
        .json::<String>()
        .await
        .unwrap()
        .contains("Failure: Error 400 Bad Request, Property notInSchema not defined in Schema"),);
}

#[test_context(TestData)]
#[tokio::test]
async fn test_cannot_create_item_with_props_defined_for_other_object(ctx: &mut TestData) {
    register_account(&ctx.pod_client).await;

    let payload = json!(
    {
        "type": "PluginRun",
        "containerImage": "",
        "targetItemId": "",
        "pluginName": "",
        // Belongs to Trigger
        "triggerOn": "Trigger like property"

    });

    let resp = ctx.pod_client.post_to(payload, "create_item").await;
    assert_eq!(resp.status(), reqwest::StatusCode::BAD_REQUEST);
    assert!(resp.json::<String>().await.unwrap().contains(
        "Failure: Error 400 Bad Request, Property triggerOn not defined in Schema for PluginRun"
    ),);
}
