mod common;
use crate::common::pod_request::{migrate_plugin_definition, register_account};
use bollard::container::ListContainersOptions;
use common::test_data::TestData;
use serde_json::json;
use std::{collections::HashMap, time::Duration};
use test_context::test_context;
use tokio::time::timeout;

#[test_context(TestData)]
#[tokio::test]
async fn test_start_plugin_by_bulk(ctx: &mut TestData) {
    register_account(&ctx.pod_client).await;

    migrate_plugin_definition(&ctx.pod_client).await;

    let payload = json!({
        "createItems": [
            {
                // Invalid plugin
                "type": "PluginRun",
                "id": "4fa2094c-51d6-4874-a135-9017fcdedf16",

                "containerImage": "NOT_EXIST.com/image:latest",
                "pluginModule": "plugin_to_trigger.plugin",
                "pluginName": "ClassifierPlugin",
                "status": "idle",
                "targetItemId": "4fa2094c-51d6-4874-a135-9017fcdedf16"

            },

            {
                // Valid plugin
                "type": "PluginRun",
                "id": "566d75e8-6163-4f04-a5eb-7a6239dc7437",

                "containerImage": "gitlab.memri.io:5050/szimnowoda/plugin_to_trigger:main-latest",
                "pluginModule": "plugin_to_trigger.plugin",
                "pluginName": "ClassifierPlugin",
                "status": "idle",
                "targetItemId": "566d75e8-6163-4f04-a5eb-7a6239dc7437"

            },

            {
                // Invalid plugin
                "type": "PluginRun",
                "id": "9139dc23-e7e8-455b-bb13-83c7b3bb3556",

                "containerImage": "not_exist.com/image:latest",
                "pluginModule": "plugin_to_trigger.plugin",
                "pluginName": "ClassifierPlugin",
                "status": "idle",
                "targetItemId": "9139dc23-e7e8-455b-bb13-83c7b3bb3556"

            }
        ]
    });

    let resp = ctx.pod_client.post_to(payload, "bulk").await;

    assert_eq!(resp.status(), reqwest::StatusCode::BAD_REQUEST);

    let mut containers = timeout(Duration::from_secs(10), async {
        let mut containers = vec![];

        while containers.is_empty() {
            tokio::time::sleep(Duration::from_secs(1)).await;

            containers = ctx
                .list_containers(ListContainersOptions {
                    filters: HashMap::from([("name", vec![ctx.owner_key.as_ref()])]),
                    ..Default::default()
                })
                .await;
        }

        containers
    })
    .await
    .unwrap();

    // There is one container running
    assert_eq!(containers.len(), 1);
    assert_eq!(containers.pop().unwrap().state, Some("running".to_string()));
}
