mod common;
use crate::common::pod_request::{
    delete_plugin, get_db_item, migrate_plugin_definition, register_account, register_trigger,
    send_fake_msg, start_plugin,
};
use common::test_data::TestData;
use test_context::test_context;

#[test_context(TestData)]
#[tokio::test]
async fn test_trigger_after_plugin_gets_deleted(ctx: &mut TestData) {
    register_account(&ctx.pod_client).await;

    migrate_plugin_definition(&ctx.pod_client).await;

    start_plugin(ctx).await;

    let trigger_id = register_trigger(&ctx.pod_client).await;

    delete_plugin(&ctx.pod_client).await;

    // Trigger should notice that plugin is deleted, and unregister itself
    send_fake_msg(&ctx.pod_client).await;

    let trigger_item = get_db_item(&trigger_id, &ctx.pod_client).await;

    let trigger_deleted = trigger_item.pointer("/deleted").unwrap().as_bool().unwrap();

    assert!(trigger_deleted);
}
