#!/bin/bash
NAMESPACE=default
NOT_FOUND=0
PORT=8080
STAGE_NAME=test
RESOURCE_NOT_EXISTS="No resources found in $NAMESPACE namespace."
PLUGINS_PODS=$(kubectl get pods -l type=plugin --field-selector=status.phase=Pending -n $NAMESPACE -o json | jq '.items[].metadata.name')
PLUGINS_RUNNING_PODS=$(kubectl get pods -l type=plugin --field-selector=status.phase=Running -n $NAMESPACE -o json | jq '.items[].metadata.name')
PLUGINS_PODS+=(${PLUGINS_RUNNING_PODS[@]})
for plugin_name in ${PLUGINS_PODS[@]}
do
    PLUGIN_NAME_ALTERED=$(echo $plugin_name | tr -d '"')
    IS_SERVICE_EXITS=$(kubectl get service --field-selector=metadata.name=$PLUGIN_NAME_ALTERED -n $NAMESPACE -o json | jq '.items | length')

    if [ $IS_SERVICE_EXITS -eq 0 ]; then
        echo "service {$PLUGIN_NAME_ALTERED} is not available"

        mkdir -p $PLUGIN_NAME_ALTERED
        cp plugin-service.yaml $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-service.yaml
        cp ingress-patch.json $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-ingress-patch.json
        # cp ingress-ssl-patch.json $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-ingress-ssl-patch.json

        sed -i "s/replace_unique_name/$PLUGIN_NAME_ALTERED/g" $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-service.yaml
        sed -i "s/replace_namespace/$NAMESPACE/g" $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-service.yaml
        sed -i "s/replace_port/$PORT/g" $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-service.yaml

        sed -i "s/replace_unique_name/$PLUGIN_NAME_ALTERED/g" $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-ingress-patch.json
        sed -i "s/replace_stage_name/$STAGE_NAME/g" $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-ingress-patch.json

        # sed -i "s/replace_unique_name/$PLUGIN_NAME_ALTERED/g" $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-ingress-ssl-patch.json
        # sed -i "s/replace_stage_name/$STAGE_NAME/g" $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-ingress-ssl-patch.json

        kubectl -n $NAMESPACE patch ingress example-ingress --type "json" -p "$(cat $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-ingress-patch.json)"
        # kubectl -n $NAMESPACE patch ingress hello-kubernetes-ingress --type "json" -p "$(cat $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-ingress-ssl-patch.json)"
        kubectl apply -f $PLUGIN_NAME_ALTERED/$PLUGIN_NAME_ALTERED-service.yaml
    fi
done

echo "all plugin pods have services configured."
