#!/bin/bash

NAMESPACE=default
NOT_FOUND=0
PORT=8080
STAGE_NAME=test
RESOURCE_NOT_EXISTS="No resources found in $NAMESPACE namespace."

PLUGINS_PODS_COMPLETED=$(kubectl get pods -l type=plugin --field-selector=status.phase!=Running -n $NAMESPACE -o json | jq '.items[].metadata.name')

for plugin_name in $PLUGINS_PODS_COMPLETED
do
    PLUGIN_NAME_ALTERED=$(echo $plugin_name | tr -d '"')
    IS_SERVICE_EXITS=$(kubectl get service --field-selector=metadata.name=$PLUGIN_NAME_ALTERED -n $NAMESPACE -o json | jq '.items | length')

    if [ $IS_SERVICE_EXITS -eq 1 ]; then
        POD_HOST='"'$PLUGIN_NAME_ALTERED.$STAGE_NAME'"'
        INDEX=$(kubectl get ingress example-ingress -n $NAMESPACE -o json | jq '.spec.rules | map(.host == '$POD_HOST') | index(true)')
        if [ "$INDEX" != "null" ]; then
            echo "Deleting $PLUGIN_NAME_ALTERED from ingress"
            kubectl patch ingress example-ingress --type=json -p="[{'op': 'remove', 'path': '/spec/rules/$INDEX'}]" -n $NAMESPACE
        fi

        SSL_INDEX=$(kubectl get ingress example-ingress -n $NAMESPACE  -o json | jq '.spec.tls[0].hosts | index('$POD_HOST')')
        if [ "$SSL_INDEX" != "null" ]; then
            echo "Deleting $PLUGIN_NAME_ALTERED from SSL Ingress"
            kubectl patch ingress example-ingress --type=json -p="[{'op': 'remove', 'path': '/spec/tls/0/hosts/$SSL_INDEX'}]" -n $NAMESPACE
        fi
        echo "Deleting service {$PLUGIN_NAME_ALTERED}"
        kubectl delete svc $PLUGIN_NAME_ALTERED -n $NAMESPACE
    fi
done

echo "Services and ingress are cleared."