"""
This file is meant to simulate flow that we have with current consumer app
There is a TT importer TASK that puts new tweets to the POD,
FrontEnd TASK, that does some small Writes, and a little bigger reads
Classifier TASK that TODO: what exactly it does?

No real plugin is created during that simulation

The parameters for the tasks are set to very pesimistic values, normally TT
importer writes once per 5 minutes for example, in the simulation we have seconds between writes
TODO: create pesimistic, and real parameters?
"""
import asyncio
import json
import os
from unicodedata import name

import aiohttp
from locust import FastHttpUser, between, events, tag, task

key = "dbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdbdb"

POD_OWNERS_COUNT = None
POD_OWNERS = None

POD_DB_DIR_ENV = "POD_DB_DIR"
CLIENT_AUTH = {"type": "ClientAuth", "databaseKey": key}
# TODO:
# use returned id, don't hardcode id
# split deploy plugin to two requests in order to create an edge


class FrontEnd(FastHttpUser):
    CURRENT_OWNER = 0

    wait_time = between(0.5, 10)

    def get_owner(self):
        owner = POD_OWNERS[FrontEnd.CURRENT_OWNER]
        FrontEnd.CURRENT_OWNER += 1
        return owner

    def on_start(self):
        self.pod_owner = self.get_owner()
        print(f"FrontEnd started for {self.pod_owner}")

    def on_stop(self):
        FrontEnd.CURRENT_OWNER -= 1

    @task(5)
    def queries_data(self):
        self.client.post("/v4/"+self.pod_owner+"/search",
                         json={
                             "auth": CLIENT_AUTH,
                             "payload": {
                                 "dateServerModified>=": 0,
                                 "[[edges]]": {},
                                 "_limit": 500,
                                 "_sortOrder": "Desc"
                             }}, name=f"{self.pod_owner} {self.__class__.__name__}.queries_data")

    @task(1)
    def some_write(self):
        self.client.post("/v4/"+self.pod_owner+"/create_item",
                         json={"auth": CLIENT_AUTH,
                               "payload": {
                                   "type": "File",
                                   "sha256": "fake sha256",
                                   "key": "fake file key",
                                   "nonce": "fake nonce"
                               }},
                         name=f"{self.pod_owner} {self.__class__.__name__}.some_write"
                         )


class TTImporter(FastHttpUser):
    CURRENT_OWNER = 0

    wait_time = between(5 * 60, 5 * 61)

    def get_owner(self):
        owner = POD_OWNERS[TTImporter.CURRENT_OWNER]
        TTImporter.CURRENT_OWNER += 1
        return owner

    def post_tweets_to_pod(self, num_messages=500):
        """
        Simulate the initial import of an importer,
        with dateReceived < trigger.filterCreatedAfter
        """
        messages = []
        for _ in range(num_messages):
            messages.append({
                "type": "Message",
                "content": "a"*500,
                "dateReceived": 1655170839991,
                "isMock": True
            })

        self.client.post("/v4/"+self.pod_owner+"/bulk", json={
            "auth": CLIENT_AUTH,
            "payload": {
                "createItems": messages,
            }
        },
            name=f"{self.pod_owner} {self.__class__.__name__}.post_tweets_to_pod")

    def on_start(self):
        """Start the plugin"""
        self.pod_owner = self.get_owner()
        self.post_tweets_to_pod()

        print(f"TT Importer started for {self.pod_owner}")

    def on_stop(self):
        print(f"TT Importer stop for {self.pod_owner}")
        TTImporter.CURRENT_OWNER -= 1
        # TODO: plugin stop??

    @task(1)
    def import_tweets(self):
        # Send a message, trigger will fire up, ping the classifier plugin to do the stuff
        self.post_tweets_to_pod(500)


class TTClassifier(FastHttpUser):
    CURRENT_OWNER = 0

    wait_time = between(5 * 60, 5 * 61)

    def get_owner(self):
        owner = POD_OWNERS[TTClassifier.CURRENT_OWNER]
        TTClassifier.CURRENT_OWNER += 1
        return owner

    def post_some_results_to_pod(self, num_messages=500):
        # TODO: make it more accurate?

        messages = []
        for _ in range(num_messages):
            messages.append({
                "type": "Message",
                "content": "a"*500,
                "dateReceived": 1655170839991,
                "isMock": True
            })

        self.client.post("/v4/"+self.pod_owner+"/bulk", json={
            "auth": CLIENT_AUTH,
            "payload": {
                "createItems": messages,
            }
        },
            name=f"{self.pod_owner} {self.__class__.__name__}.post_tweets_to_pod")

    def on_start(self):
        """Start the plugin"""
        self.pod_owner = self.get_owner()
        self.post_some_results_to_pod()

        print(f"TTClassifier started for {self.pod_owner}")

    def on_stop(self):
        print(f"TTClassifier stop for {self.pod_owner}")
        TTClassifier.CURRENT_OWNER -= 1
        # TODO: plugin stop??

    @task(1)
    def classify_feed(self):
        # Send a message, trigger will fire up, ping the classifier plugin to do the stuff
        self.post_some_results_to_pod(1000)


async def post_bulk(session, base_url, owner, payload):
    url = base_url + "/v4/" + owner + "/bulk"

    async with session.request("POST", url,
                               json={"auth": CLIENT_AUTH, "payload": payload}) as resp:
        return (resp.status, url)


async def create_accounts(base_url, session):
    tasks = []
    for owner in POD_OWNERS:
        tasks.append(asyncio.create_task(create_new_account(owner, base_url, session)))

    _responses = await asyncio.gather(*tasks)


async def create_new_account(owner, base_url, session):
    url = base_url + "/v4/account"

    async with session.request("POST", url,
                               json={"ownerKey": owner, "databaseKey": key}) as resp:
        return (resp.status, url)


async def flutter_migration(base_url, session):
    """Fills the database with the schema definitions from the frontend"""
    FLUTTER_MIGRATION = "./flutter_migration"
    for bulk in sorted(os.listdir(FLUTTER_MIGRATION)):
        print(f"bulk {bulk}")
        with open(os.path.join(FLUTTER_MIGRATION, bulk), 'r') as file:
            payload = json.loads(file.read())

            tasks = []
            for owner in POD_OWNERS:
                tasks.append(asyncio.create_task(post_bulk(session, base_url, owner, payload)))

            _responses = await asyncio.gather(*tasks)


async def fill_db(base_url):
    """Create DB for each POD owner with some data
    note each DB makes POD to allocate at least that many FDs:
    opened_fds = nr_of_cores * 3 (.sqlite, -shm. -wal files) + at least one TCP connection from locust

    Please keep it in mind when launching with default linux FD limit set to 1024
    """

    async with aiohttp.ClientSession() as session:
        await create_accounts(base_url, session)
        await flutter_migration(base_url, session)


@events.test_start.add_listener
def on_test_start(environment, **kwargs):
    global POD_OWNERS_COUNT
    global POD_OWNERS

    num_users = environment.parsed_options.num_users

    if not num_users:
        print("Please pass num users as an parameter to locust CLI")
        os.abort()

    user_classes = environment.user_classes

    print(f"user_classes: {user_classes}")

    # The num_users passed as a parameter to locust does not correspond to the number of POD owners
    # Divide num_users by number of locust client class that are taken to the simulation
    POD_OWNERS_COUNT = num_users // len(user_classes)

    # Note beef has to be lowercase, otherwise kubernetes container will not run
    POD_OWNERS = ["beef%05d" %
                  i for i in range(POD_OWNERS_COUNT)]

    # Set exactly one FrontEnd and one ClassifierPlugin per POD owner
    # fixed_count cannot be greater than num_users
    FrontEnd.fixed_count = POD_OWNERS_COUNT
    TTClassifier.fixed_count = POD_OWNERS_COUNT
    TTImporter.fixed_count = POD_OWNERS_COUNT

    print(f"A new test for {environment.host} is starting. Fill up the DB...")
    asyncio.run(fill_db(environment.host))


@events.test_stop.add_listener
def on_test_stop(environment, **kwargs):
    pass

    # if not os.getenv(POD_DB_DIR_ENV):
    #     print(f"{POD_DB_DIR_ENV} env not set, cleanup aborted")
    #     return

    # DB_SUFFIX = ".p4.4.sqlite"

    # print(f"A test for {environment.host} is ending, cleaning up databases...")

    # db_dir = os.getenv(POD_DB_DIR_ENV)

    # for owner in POD_OWNERS:
    #     db_path = os.path.join(db_dir, owner)

    #     try:
    #         os.remove(db_path + DB_SUFFIX)
    #     except Exception as e:
    #         pass

    #     try:
    #         os.remove(db_path + DB_SUFFIX + "-wal")
    #     except Exception as e:
    #         pass

    #     try:
    #         os.remove(db_path + DB_SUFFIX + "-shm")
    #     except Exception as e:
    #         pass
